@@ -2,12 +2,11 @@ package Pcore::HTTP;
 
 use Pcore -const,
   -export => {
-    METHODS    => [qw[http_acl http_baseline_control http_bind http_checkin http_checkout http_connect http_copy http_delete http_get http_head http_label http_link http_lock http_merge http_mkactivity http_mkcalendar http_mkcol http_mkredirectref http_mkworkspace http_move http_options http_orderpatch http_patch http_post http_pri http_propfind http_proppatch http_put http_rebind http_report http_search http_trace http_unbind http_uncheckout http_unlink http_unlock http_update http_updateredirectref http_version_control]],
-    TLS_CTX    => [qw[$TLS_CTX_HIGH $TLS_CTX_LOW]],
-    PERSISTENT => [qw[$PERSISTENT_IDENT $PERSISTENT_ANY $PERSISTENT_NO_PROXY]],
+    METHODS => [qw[http_acl http_baseline_control http_bind http_checkin http_checkout http_connect http_copy http_delete http_get http_head http_label http_link http_lock http_merge http_mkactivity http_mkcalendar http_mkcol http_mkredirectref http_mkworkspace http_move http_options http_orderpatch http_patch http_post http_pri http_propfind http_proppatch http_put http_rebind http_report http_search http_trace http_unbind http_uncheckout http_unlink http_unlock http_update http_updateredirectref http_version_control]],
+    TLS_CTX => [qw[$TLS_CTX_HIGH $TLS_CTX_LOW]],
   };
 use Pcore::Util::Scalar qw[blessed is_glob];
-use Pcore::AE::Handle qw[:PERSISTENT];
+use Pcore::AE::Handle2 qw[:TLS_CTX];
 use Pcore::HTTP::Util;
 use Pcore::HTTP::Headers;
 use Pcore::HTTP::Response;
@@ -20,62 +19,40 @@ use Pcore::HTTP::Cookies;
 # 598 - user aborted request via on_header or on_body.
 # 599 - other, usually nonretryable, errors (garbled URL etc.).
 
-const our $TLS_CTX_LOW  => 1;
-const our $TLS_CTX_HIGH => 2;
-const our $TLS_CTX      => {
-    $TLS_CTX_LOW => {
-        ca_file         => P->ca->ca_file,
-        cache           => 1,
-        verify          => 0,
-        verify_peername => undef,
-        sslv2           => 1,
-        dh              => undef,            # Diffie-Hellman is disabled
-    },
-    $TLS_CTX_HIGH => {
-        ca_file         => P->ca->ca_file,
-        cache           => 1,
-        verify          => 1,
-        verify_peername => 'http',
-        sslv2           => 0,
-        dh              => 'schmorp4096',
-    },
-};
-
 our $DEFAULT = {
     method => undef,
     url    => undef,
 
-    useragent         => "Pcore-HTTP/$Pcore::VERSION",
-    recurse           => 7,                              # max. redirects
-    keepalive_timeout => undef,                          # keepalive timeout for persistent connections, if false - default value will be used
-    timeout           => 300,                            # timeout in seconds
-    connect_timeout   => undef,                          # handle socket connect timeout
-    accept_compressed => 1,                              # add ACCEPT_ENCODIING header
-    decompress        => 1,                              # automatically decompress
-    persistent        => $PERSISTENT_IDENT,
-    session           => undef,
-    cookies           => undef,                          # 1 - create temp cookie jar object, HashRef - use as cookies storage
+    useragent => "Pcore-HTTP/$Pcore::VERSION",    # shortcut for User-Agent header
+    recurse   => 7,                               # max. redirects
 
-    # write body to fh if body length > this value, 0 - always store in memory, 1 - always store to file
-    buf_size => 0,
+    accept_compressed => 1,                       # add ACCEPT_ENCODIING header
+    decompress        => 1,                       # automatically decompress
 
-    tls_ctx       => $TLS_CTX_LOW,
-    bind_ip       => undef,
-    proxy         => undef,
-    handle_params => undef,
+    cookies => undef,                             # 1 - create temp cookie jar object, HashRef - use as cookies storage
+
+    persistent => 0,                              # persistent timeout in seconds, proxy connection can't be persistent
+
+    buf_size => 0,                                # write body to fh if body length > this value, 0 - always store in memory, 1 - always store to file
+
+    handle_params   => undef,                     # HashRef with params, that will be passed directly to AE::Handle
+    connect_timeout => undef,                     # handle connect timeout
+    timeout         => 300,                       # timeout in seconds
+    tls_ctx         => $TLS_CTX_HIGH,
+    bind_ip         => undef,
+    proxy           => undef,
 
     headers => undef,
     body    => undef,
 
-    # 1 - create progress indicator, HashRef - progress indicator params, CodeRef - on_progress callback
-    on_progress   => undef,
+    on_progress   => undef,                       # 1 - create progress indicator, HashRef - progress indicator params, CodeRef - on_progress callback
     on_header     => undef,
     on_body       => undef,
     before_finish => undef,
     on_finish     => undef,
 };
 
-our $DEFAULT_HANDLE_PARAMS = {    #
+our $DEFAULT_HANDLE_PARAMS = {                    #
     max_read_size => 1_048_576,
 };
 
@@ -186,6 +163,9 @@ sub request ( @ ) {
 
     $args{url} = P->uri( $args{url}, base => 'http://', authority => 1 ) if !ref $args{url};
 
+    # proxy connections can't be persistent
+    $args{persistent} = 0 if $args{proxy};
+
     # create headers object
     if ( !$args{headers} ) {
         $args{headers} = Pcore::HTTP::Headers->new;
@@ -241,9 +221,6 @@ sub request ( @ ) {
         $args{headers}->{USER_AGENT} = $useragent if !exists $args{headers}->{USER_AGENT};
     }
 
-    # resolve TLS context shortcut
-    $args{tls_ctx} = $TLS_CTX->{ $args{tls_ctx} } if !ref $args{tls_ctx};
-
     # resolve on_progress shortcut
     if ( $args{on_progress} && ref $args{on_progress} ne 'CODE' ) {
         if ( !ref $args{on_progress} ) {
@@ -412,11 +389,11 @@ sub _get_on_progress_cb (%args) {
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ## | Sev. | Lines                | Policy                                                                                                         |
 ## |======+======================+================================================================================================================|
-## |    3 | 129                  | ErrorHandling::RequireCheckingReturnValueOfEval - Return value of eval not tested                              |
+## |    3 | 106                  | ErrorHandling::RequireCheckingReturnValueOfEval - Return value of eval not tested                              |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    3 | 184                  | Subroutines::ProhibitExcessComplexity - Subroutine "request" with high complexity score (31)                   |
+## |    3 | 161                  | Subroutines::ProhibitExcessComplexity - Subroutine "request" with high complexity score (31)                   |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    2 | 170                  | ValuesAndExpressions::ProhibitLongChainsOfMethodCalls - Found method-call chain of length 4                    |
+## |    2 | 147                  | ValuesAndExpressions::ProhibitLongChainsOfMethodCalls - Found method-call chain of length 4                    |
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ##
 ## -----SOURCE FILTER LOG END-----
